"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const error_1 = __importDefault(require("./error"));
class Model {
    static async init({ wechat, modelName }) {
        this.wechat = wechat;
        this.modelName = modelName || this.name.toLowerCase();
        this.accessToken = await wechat.getAccessToken();
        this.wechat.addModel(this);
    }
    static get request() {
        const instance = axios_1.default.create({
            baseURL: 'https://api.weixin.qq.com/cgi-bin',
            params: {
                access_token: this.accessToken
            }
        });
        instance.interceptors.response.use(async (res) => {
            const data = res.data;
            if (data.errcode) {
                // 表示 token 过期
                if (data.errcode === 42001) {
                    await this.wechat.set('');
                }
                throw new error_1.default(data.errmsg, data.errcode, {
                    response: res
                });
            }
            return res;
        });
        return instance;
    }
}
exports.default = Model;
