"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const article_1 = require("../models/article");
const news_1 = require("../models/news");
const DEFAULT_GET_TOKEN = () => '';
const DEFAULT_SET_TOKEN = (token) => { };
async function createModel(cls, wechat) {
    class ExtendModel extends cls {
    }
    await ExtendModel.init({ wechat, modelName: cls.name.toLowerCase() });
    return ExtendModel;
}
class Wechat {
    constructor(appId, appSecrect, get = DEFAULT_GET_TOKEN, set = DEFAULT_SET_TOKEN) {
        this.appId = appId;
        this.appSecret = appSecrect;
        this.get = get;
        this.set = set;
        this.models = {};
    }
    async _getAccessToken() {
        const { data } = await axios_1.default.request({
            url: 'https://api.weixin.qq.com/cgi-bin/token',
            params: {
                appId: this.appId,
                secret: this.appSecret,
                grant_type: 'client_credential'
            }
        });
        if (data.errorcode) {
            throw new Error(data.errormsg);
        }
        return data.access_token;
    }
    async sync() {
        await createModel(article_1.Article, this);
        await createModel(news_1.News, this);
    }
    async getAccessToken() {
        let token;
        try {
            token = await this.get();
        }
        catch (e) {
        }
        if (!token) {
            const refreshToken = await this._getAccessToken();
            await this.set(refreshToken);
            return refreshToken;
        }
        return token;
    }
    addModel(model) {
        this.models[model.modelName] = model;
    }
}
exports.default = Wechat;
