"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Article = void 0;
const p_map_1 = __importDefault(require("p-map"));
const lodash_1 = __importDefault(require("lodash"));
const axios_1 = __importDefault(require("axios"));
const path_1 = __importDefault(require("path"));
const model_1 = __importDefault(require("../lib/model"));
const formstream = require('formstream');
class Article extends model_1.default {
    constructor() {
        super(...arguments);
        this.title = '';
        this.thumbMediaId = '';
        this.content = 'Powerd by weichat-api';
        this.showCoverPic = 1;
        this.needOpenComment = 1;
        this.onlyFansCanComment = 1;
    }
    static async _create(article) {
        const articles = Array.isArray(article) ? article : [article];
        const prepareArticles = await p_map_1.default(articles, async (article) => {
            const content = await this.prepareContent(article.content);
            console.log(content);
            return {
                ...lodash_1.default.mapKeys(article, (value, key) => lodash_1.default.snakeCase(key)),
                content
            };
        });
        return this.request({
            url: '/material/add_news',
            method: 'POST',
            params: {
                access_token: this.accessToken
            },
            data: {
                articles: prepareArticles
            }
        });
    }
    static async uploadImage(src) {
        const { data: buffer } = await axios_1.default({
            url: src,
            responseType: 'arraybuffer'
        });
        // FormData cant work
        // const form = new FormData()
        // form.append('media', fs.createReadStream(path.resolve(__dirname, 'hello.jpg')), {
        //   knownLength: 29506,
        //   contentType: 'image/jpeg',
        //   filename: 'hello.jpg'
        // })
        const form = formstream();
        form.buffer('media', buffer, path_1.default.basename(src));
        const { data } = await this.request.post('/media/uploadimg', form, {
            headers: form.headers(),
            params: {
                access_token: this.accessToken
            }
        });
        return data.url;
    }
    static async prepareContent(content) {
        const re = /<img.*?src="(.*?)".*?>/g;
        const match = content.matchAll(re);
        const imgs = Array.from(match, x => x[1]);
        // 批量上传图片
        const imgList = await p_map_1.default(lodash_1.default.uniq(imgs), async (src) => {
            const weixinImg = src.includes('mmbiz') ? src : await this.uploadImage(src);
            return { src, weixinImg };
        }, { concurrency: 3 });
        const imgMap = lodash_1.default.keyBy(imgList, 'src');
        return content.replace(new RegExp(imgs.join('|')), (src) => {
            var _a;
            const weixinSrc = ((_a = imgMap[src]) === null || _a === void 0 ? void 0 : _a.weixinImg) || src;
            return weixinSrc;
        });
    }
    static create(article) {
        return this._create(article);
    }
    static bulkCreate(articles) {
        return this._create(articles);
    }
}
exports.Article = Article;
